/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skArticleInfoDialog.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skArticleInfoDialog.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <q3textedit.h>
#include <q3hbox.h>
#include <q3groupbox.h>
#include <qpainter.h>
#include <qprinter.h>
#include <q3paintdevicemetrics.h>
#include <q3simplerichtext.h>
#include <qrect.h>
//Added by qt3to4:
#include <Q3VBoxLayout>

/*------------------------------------------------------ Class functions */

/** The constructor for the modal \a skArticleInfoDialog class. 
*/
skArticleInfoDialog::skArticleInfoDialog(const skSegorArticle &article,
                                         QWidget *parent, 
                                         QString name )
    : QDialog( parent, name, true)
    
{
    currentArticle = article;
    Q3BoxLayout *vb = new Q3VBoxLayout( this );

    Q3GroupBox *gbArticleInfo = new Q3GroupBox(1, Qt::Horizontal, tr("Artikel-Info"), this);
    teArticleInfo = new Q3TextEdit( gbArticleInfo, "teArticleInfo" );
    teArticleInfo->setReadOnly(true);
    Q3HBox *hbButtons = new Q3HBox( this );
    QPushButton *print = new QPushButton( tr("Drucken"), hbButtons, "pbPrint" );
    QPushButton *ok = new QPushButton( tr("OK"), hbButtons, "pbOK" );
    connect( print, SIGNAL(clicked()), SLOT(slotPrint()) );
    connect( ok, SIGNAL(clicked()), SLOT(accept()) );

    vb->addWidget(gbArticleInfo);
    vb->addWidget(hbButtons);
    vb->activate();

    print->setFixedSize(print->width(), print->height());
    ok->setFixedSize(ok->width(), ok->height());

    resize(550, 400);

    QString text;
    getInfoText(currentArticle, text);
    teArticleInfo->setText(text);
}

/** Returns an info string \a s that contains the data of
* \a article in RichText/HTML format.
*/
void skArticleInfoDialog::getInfoText(const skSegorArticle &article,
                                      QString &s)
{
  s = "<QT><CENTER><TABLE>\n";
  
  
  // Name
  s += "<TR><TD><B>";
  s += tr("Name") + "</B>:</TD><TD>";
  s += QString(article.Name.c_str());
  s += "</TD></TR>\n";
  
  // Description
  s += "<TR><TD><B>";
  s += tr("Beschreibung") + "</B>:</TD><TD>";
  s += QString(article.Description.c_str());
  s += "</TD></TR>\n";
 
  // Other names
  s += "<TR><TD><B>";
  s += tr("Weitere Namen") + "</B>:</TD><TD>";
  s += QString(article.OtherNames.c_str());
  s += "</TD></TR>\n";

   // Status
  s += "<TR><TD><B>";
  s += tr("Status") + "</B>:</TD><TD>";
  switch (article.Status)
  {    
    case 1:  s += tr("Lagertyp, wird nach Mglichkeit stets ab Lager angeboten "
                     "(wenn auch vielleicht nicht immer in der gewnschten Menge)");
             s += ".";
             break;
    case 2:  s += "<B>R</B> = ";
             s += tr("Restbestand im Lager, wenn dieser verkauft ist erhlt der Artikel "
                     "automatisch den Status B (Bestellung erforderlich)");
             s += ".";
             break;
    case 3:  s += "<B>B</B> = ";
             s += tr("Bestellung erforderlich, der Artikel wird nur auf Kundenwunsch bestellt");
             s += ".";
             break;
    case 4:  s += "<B>A</B> = ";
             s += tr("Ausverkauf, wenn der aktuelle Lagerbestand alle ist kann der Artikel nicht mehr "
                     "bzw. nur noch zum regulren Marktpreis (z.B. bei Sonderposten) geliefert werden");
             s += ".";
             break;
    default: s += tr("Unbekannt");
             break;
 
  }
  s += "</TD></TR>\n";
 
  // Order unit
  s += "<TR><TD><B>";
  s += tr("1 VPE enthlt") + "</B>:</TD><TD>";
  s += QString::number(article.PerUnit);
  s += QString(" ") + tr("Stck");
  s += "</TD></TR>\n";

  // Prices
  s += "<TR><TD><B>";
  s += tr("Preise fr ");
  if (1 == article.UnitPriceKey)
    s += "1 ";
  else
   s += "100 ";
  s += tr("Stck");
  s += "</B>:</TD><TD>";

  if (article.plPrices.size() > 0)
  {
    // Construct table with prices
    s += "<TABLE>\n";
    s += "<TR><TH ALIGN=CENTER>";
    s += tr("Menge");
    s += "</TH><TH ALIGN=CENTER>";
    s += tr("Netto");
    s += "</TH><TH ALIGN=CENTER>";
    s += tr("Brutto");
    s += "<BR>(incl. ";
    if (1 == article.AVTaxKey)
      s += "16%)";    
    else
      s += "7%)";    
    s += "</TH></TR>\n";

    unsigned int pCnt = 0;
    for (; pCnt < article.plPrices.size(); ++pCnt)
    {
      s += "<TR><TD ALIGN=RIGHT>";
      s += QString::number(article.plPrices[pCnt].Units);
      s += "</TD><TD ALIGN=RIGHT>";
      if (1 == article.AVTaxKey)
        s += QString::number(article.plPrices[pCnt].Price/1.16, 'f', 2);
      else
        s += QString::number(article.plPrices[pCnt].Price/1.07, 'f', 2);
      s += "</TD><TD ALIGN=RIGHT>";
      s += QString::number(article.plPrices[pCnt].Price, 'f', 2);
      s += "</TD><TR>\n";
    }   

    s += "</TABLE>\n";

  }
  else
  {
    s += "Keine Preisinformationen vorhanden! Bitte fragen Sie bei Segor electronics nach.";
  }
  s += "</TD></TR>\n";

  s += "</TABLE></CENTER></QT>\n";
}

/*--------------------------------------------------------------- Slots */

/** Prints the current article infos.
*/
void skArticleInfoDialog::slotPrint()
{
  QPrinter printer;
  printer.setFullPage(TRUE);
  if ( printer.setup( this ) ) 
  {
    QPainter p( &printer );
    Q3PaintDeviceMetrics metrics(p.device());
    int dpix = metrics.logicalDpiX();
    int dpiy = metrics.logicalDpiY();
    const int margin = 72; // pt
    QRect body(margin*dpix/72, margin*dpiy/72,
               metrics.width()-margin*dpix/72*2,
               metrics.height()-margin*dpiy/72*2 );

    QString rText;
    // Create rich text
    getInfoText(currentArticle, rText);

    Q3SimpleRichText richText( rText, QFont());
    richText.setWidth( &p, body.width() );
    QRect view( body );
    int page = 1;
    do 
    {
      richText.draw( &p, body.left(), body.top(), view, colorGroup() );
      view.moveBy( 0, body.height() );
      p.translate( 0 , -body.height() );
      p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
                  view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
      if ( view.top()  >= richText.height() )
          break;
      printer.newPage();
      page++;
    } while (TRUE);
  }

}

