/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skBitParser.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skBitParser.h"
#include "skFileIO.h"

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skBitParser class.
*/
skBitParser::skBitParser() : 
	currentByte(0),
        remainingBits(0),
        readBytes(0)
{
  ;
}

/** Resets the bit parser such that on the next call
* to \a getBit a new byte is read from the file.
*/
void skBitParser::reset()
{
  remainingBits = 0;
  readBytes = 0;
}

/** Returns the number of read bytes.
* @return Number of bytes read since last reset
*/
unsigned long int skBitParser::count()
{
  return readBytes;
}

/** Returns the number of bits that can still be read
* for the current byte.
* @return Number of available bits
*/
int skBitParser::bits()
{
  return remainingBits;
}

/** Reads a single bit from the given file stream \a f.
* @param f File stream, the entries are read from
* @return The next bit (`1' or `0')
*/
int skBitParser::getBit(ifstream &f)
{
  if (remainingBits > 0)
  {
    // Shift current byte up
    currentByte = (currentByte << 1) & 0xFF;
    --remainingBits;
  }
  else
  {
    // Read new byte from file
    currentByte = readUChar(f);
    remainingBits = 7;
    ++readBytes;
  }

  // Extract upper bit
  if (0x80 == (currentByte & 0x80))
    return 1;
  else
    return 0;
}

