/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skFileIO.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skFileIO.h"

/*------------------------------------------------------------ Functions */

/** Reads an unsigned char from the file stream \a f.
* @param f File where the char is read from
* @return The unsigned char
*/
unsigned char readUChar(ifstream &f)
{
  char byte;

  f.get(byte);

  return ((unsigned char) byte);
}

/** Writes an unsigned char to the file stream \a f.
* @param f File the char is written to
* @param byte The unsigned char
*/
void writeUChar(ofstream &f, unsigned char byte)
{
  f.put((char) byte);
}

/** Reads a signed int, consisting of 2 bytes
* from the file stream \a f.
* @param f File where the int is read from
* @return The signed int
*/
int readInt(ifstream &f)
{
  unsigned char a = readUChar(f);
  unsigned char b = readUChar(f);

  // Is it a negative number?
  if (0x80 == (b & 0x80))
  {
    // Yes, so convert the 2s complement number
    // with sign bit to an int...

    // First, invert both chars bitwise (-> sign bit gets zero
    // and the number positive)...
    b = ~b;
    a = ~a;
    // ... then combine MSB and LSB, add a `1' (strips off the
    // 2s complement completely), and negate the integer.
    return ((((int) b)*256 + ((int) a) + 1)*(-1));
  }
  else
  {
    // Simply combine MSB and LSB for positive ints.
    return  (((int) b)*256 + ((int) a));
  }

}

/** Reads an unsigned long int, consisting of 4 bytes
* from the file stream \a f.
* @param f File where the int is read from
* @return The signed int
*/
unsigned long int readULongInt(ifstream &f)
{
  unsigned char c;
  unsigned long int result;

  c = readUChar(f);
  result = (unsigned long int) c;
  c = readUChar(f);
  result |= ((unsigned long int) c) << 8;
  c = readUChar(f);
  result |= ((unsigned long int) c) << 16;
  c = readUChar(f);
  result |= ((unsigned long int) c) << 24;

  return result;
}

/** Removes leading and trailing whitespace from
* the given string \a s.
* @param s String that is cleaned up
*/
void simplifyWhiteSpace(string &s)
{
  if (0 == s.size())
    return;     // String is empty

  string::size_type pos = 0;
  string::size_type start = 0;

  // Remove leading whitespace
  start = s.find_first_of(" \r\n\t");
  if (0 == start)
  { 
    pos = s.find_first_not_of(" \r\n\t", start);
    s = s.replace(start, pos - start, "");
  }

  pos = 0;
  start = 0;
  // Replace whitespace with a single blank
  while (string::npos != start)
  {
    start = s.find_first_of(" \r\n\t", pos);
    if (string::npos != start)
    {
      pos = s.find_first_not_of(" \r\n\t", start);
      s = s.replace(start, pos - start, " ");
      if (string::npos != pos)
        pos = start + 2;
      else
        start = string::npos;
    }
  }

  // Remove trailing whitespace (which collapsed to a 
  // single blank by now)
  if ((s.size() > 0) && (' ' == s[s.size()-1]))
    s = s.replace(s.size()-1, 1, "");
}

