/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorCategory.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#ifdef SEGORKAT_QT
  #include <qapplication.h>
  #include <q3listview.h>
  #include <qcombobox.h>
  #include <qstring.h>
#endif

#include "skGlobal.h"
#include "skSegorCategory.h"
#include "skFileIO.h"

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skSegorCategory class.
*/
skSegorCategory::skSegorCategory() : 
	ID(),
        Description()
{
  ;
}

/** Copy constructor for the \a skSegorCategory class.
* @param elem Category that is copied
*/
skSegorCategory::skSegorCategory(const skSegorCategory &elem) 
{
  ID = elem.ID;
  Description = elem.Description;

  sclCategories.clear();
  vector<skSegorSubCategory>::const_iterator c_it = elem.sclCategories.begin();
  for (; c_it != elem.sclCategories.end(); ++c_it)
    sclCategories.push_back(*c_it);
  
}

/** Overloaded copy operator ``=''.
* @param elem Category that is copied
*/
skSegorCategory &skSegorCategory::operator= (const skSegorCategory &elem)
{
  if (this == &elem)
    return *this;

  ID = elem.ID;
  Description = elem.Description;

  sclCategories.clear();
  vector<skSegorSubCategory>::const_iterator c_it = elem.sclCategories.begin();
  for (; c_it != elem.sclCategories.end(); ++c_it)
    sclCategories.push_back(*c_it);

  return *this;
}

/** Reads the single entries from the given file stream \a f.
* @param f File stream, the entries are read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorCategory::read(ifstream &f)
{
  char c;

  sclCategories.clear();
  ID = "";
  Description = "";

  // Read ID
  f.get(c);
  ID += c;
  // Read description
  int i;
  bool finished = false;
  for (i = 0; i < 40; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Description += c;
    else
      finished = true;
  }
  simplifyWhiteSpace(Description);

  // Skip additional index
  f.get(c);
  f.get(c);

  return OK;
}

/** Appends the given \a skSegorSubCategory to the end
* of the list.
* @param category The category to append
*/
void skSegorCategory::InsertSubCategory(const skSegorSubCategory &category)
{
  sclCategories.push_back(category);
}

/** Appends the given \a skSegorSubSubCategory to the proper
* subcategory.
* @param category The category to append
*/
void skSegorCategory::InsertSubSubCategory(const skSegorSubSubCategory &category)
{
  // Extract ID
  string firstIDs = category.ID.substr(0, 3);
  // Scan subcategories for proper ID
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
  {
    if ((*c_it).ID == firstIDs)
    {
      c_it->InsertSubSubCategory(category);
      return;
    }
  }
}

/** Appends the given \a skSegorArticle to the proper subcategory.
* @param article The article to append
*/
void skSegorCategory::InsertArticle(const skSegorArticle &article)
{
  // Extract ID
  string firstIDs = article.parentID.substr(0, 3);
  // Scan subcategories for proper ID
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
  {
    if ((*c_it).ID == firstIDs)
    {
      c_it->InsertArticle(article);
      return;
    }
  }
}


#ifdef SEGORKAT_QT

/** Displays all articles from the first subsub-category 
* in the display list. Additionally, resets the entries in the
* combo boxes \a cbSubCategory and \a cbSubSubCategory. 
* This function is called when the combo box for a 
* category changed.
*/
void skSegorCategory::UpdateCategory(Q3ListView *lvParts,
                                     QComboBox *cbSubCategory,
                                     QComboBox *cbSubSubCategory,
                                     int categoryIndex)
{
  cbSubCategory->clear();

  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
    cbSubCategory->insertItem(QString(c_it->Description.c_str()));

  cbSubCategory->setCurrentItem(0);
  sclCategories[0].UpdateSubCategory(lvParts, cbSubSubCategory, categoryIndex, 0);
}

/** Displays all articles from subcategory \a subCategoryIndex
* in the display list. Additionally, resets the entries in the
* combo box \a cbSubSubCategory. This function is called when
* the combo box for a subcategory changed.
*/
void skSegorCategory::UpdateSubCategory(Q3ListView *lvParts,
                                        QComboBox *cbSubSubCategory,
                                        int categoryIndex,
                                        int subCategoryIndex)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
  {
    lvParts->clear();
    return;
  }

  sclCategories[subCategoryIndex].UpdateSubCategory(lvParts,
                                                    cbSubSubCategory,
                                                    categoryIndex,
                                                    subCategoryIndex);
}

/** Displays all articles from subsub-category \a subsubCategoryIndex
* in the display list.
*/
void skSegorCategory::UpdateSubSubCategory(Q3ListView *lvParts,
                                           int categoryIndex,
                                           int subCategoryIndex,
                                           int subsubCategoryIndex)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
  {
    lvParts->clear();
    return;
  }

  sclCategories[subCategoryIndex].UpdateSubSubCategory(lvParts,
                                                       categoryIndex,
                                                       subCategoryIndex,
                                                       subsubCategoryIndex);
}

/** Updates the entries of the single article \a articleIndex
* in the display list. This function is called when the number of ordered
* units changed, for example.
*/
void skSegorCategory::UpdateArticleItem(Q3ListViewItem *lviItem,
                                        int categoryIndex,
                                        int subCategoryIndex,
                                        int subsubCategoryIndex,
                                        int articleIndex)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
    return;

  sclCategories[subCategoryIndex].UpdateArticleItem(lviItem,
                                                    categoryIndex,
                                                    subCategoryIndex,
                                                    subsubCategoryIndex,
                                                    articleIndex);
}

/** Scans through the subcategories and adds all articles to the
* display list that match the given search specs.
* @param search The regular expression to search for
* @inName Is ``true'' if the name may match the expr, ``false'' else
* @inDescription Is ``true'' if the description may match the expr, ``false'' else
* @inOtherName Is ``true'' if one of the other article
*              names may match the expr, ``false'' else
* @caseSensitive Is ``true'' if the search is case sensitive, ``false'' else
*/
void skSegorCategory::SearchArticles(Q3ListView *lvParts,
                                     const QString &search,
                                     bool inName,
                                     bool inDescription,
                                     bool inOtherNames,
                                     bool caseSensitive,
                                     int categoryIndex)
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  int subcategoryIndex = 0;
  for (; c_it != sclCategories.end(); ++c_it, ++subcategoryIndex)
    c_it->SearchArticles(lvParts, search, inName, inDescription,
                         inOtherNames, caseSensitive, categoryIndex, subcategoryIndex);

}

/** Opens the ``Article Info'' dialog for the article \a articleIndex in
* sub-category \a subCategoryIndex and subsub-category \a subsubCategoryIndex.
*/
void skSegorCategory::ShowArticleInfo(int subCategoryIndex,
                                      int subsubCategoryIndex,
                                      int articleIndex)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
    return;

  sclCategories[subCategoryIndex].ShowArticleInfo(subsubCategoryIndex, articleIndex);
}

/** Opens the ``Order'' dialog for the article \a articleIndex in
* sub-category \a subCategoryIndex and subsub-category \a subsubCategoryIndex.
*/
void skSegorCategory::OrderArticle(int subCategoryIndex,
                                   int subsubCategoryIndex,
                                   int articleIndex)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
    return;

  sclCategories[subCategoryIndex].OrderArticle(subsubCategoryIndex, articleIndex);
}

/** Adds all ordered articles for this category to the
* article display list.
*/
void skSegorCategory::OrderedArticles(Q3ListView *lvParts,
                                      int categoryIndex)
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  int subcategoryIndex = 0;
  for (; c_it != sclCategories.end(); ++c_it, ++subcategoryIndex)
    c_it->OrderedArticles(lvParts, categoryIndex, subcategoryIndex);

}

/** Compiles a RichText/HTML format order list by scanning through
* all subcategories.
*/
void skSegorCategory::getHtmlOrderTable(QString &sTable)
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
    c_it->getHtmlOrderTable(sTable);

}
 
/** Compiles a Text format order list by scanning through
* all subcategories.
*/
void skSegorCategory::getTextOrderTable(QString &sTable,
                                        const QString &sepString, 
                                        int ColCountWidth, 
                                        int ColArticleWidth, 
                                        int ColNettoWidth,
                                        int ColBruttoWidth)
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
    c_it->getTextOrderTable(sTable, sepString, ColCountWidth, ColArticleWidth,
                            ColNettoWidth, ColBruttoWidth);
}

#endif // of: #ifdef SEGORKAT_QT ...

/** Orders the article \a articleIndex by \a orderedUnits
* times, if the latter is zero this means the article
* is removed from the order list. 
*/
void skSegorCategory::SetOrderArticle(int subCategoryIndex,
                                      int subsubCategoryIndex,
                                      int articleIndex,
                                      int orderedUnits)
{
  if ((unsigned int) subCategoryIndex >= sclCategories.size())
    return;

  sclCategories[subCategoryIndex].SetOrderArticle(subsubCategoryIndex, articleIndex, orderedUnits);
}

/** Removes all articles from the order list.
*/
void skSegorCategory::DisOrderAll()
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
    c_it->DisOrderAll();
}

/** Updates the price sums and the number of ordered articles
* by scanning through all subcategories.
*/
void skSegorCategory::UpdateOrderInfo(int *articleCount,
                                      double *nettoSum,
                                      double *bruttoSum)
{
  vector<skSegorSubCategory>::iterator c_it = sclCategories.begin();
 
  for (; c_it != sclCategories.end(); ++c_it)
    c_it->UpdateOrderInfo(articleCount, nettoSum, bruttoSum);
}



