/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorPrice.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skGlobal.h"
#include "skSegorPrice.h"
#include "skFileIO.h"

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skSegorPrice class.
*/
skSegorPrice::skSegorPrice() : 
        Units(0),
	Price(0.0)
{
  ;
}

/** Copy constructor for the \a skSegorPrice class.
* @param elem File info that is copied
*/
skSegorPrice::skSegorPrice(const skSegorPrice &elem) 
{
  Units = elem.Units;
  Price = elem.Price;
}

/** Overloaded copy operator ``=''.
* @param elem File info that is copied
*/
skSegorPrice &skSegorPrice::operator= (const skSegorPrice &elem)
{
  if (this == &elem)
    return *this;

  Units = elem.Units;
  Price = elem.Price;

  return *this;
}

/** Reads a single units entry from the given file stream \a f.
* @param f File stream, the entry is read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorPrice::readUnits(ifstream &f)
{
  // Read number of units
  Units = readULongInt(f);

  return OK;
}

/** Reads a single price entry from the given file stream \a f.
* @param f File stream, the entry is read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorPrice::readPrice(ifstream &f)
{
  // Read price
  Price = ((double) readULongInt(f))/1000.0;

  return OK;
}


