/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programming/cpp/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorSubSubCategory.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#ifdef SEGORKAT_QT
  #include <qstring.h>
  #include <qregexp.h>
  #include <qcombobox.h>
  #include <q3listview.h>
  #include <qpixmap.h>

  #include "skArticleInfoDialog.h"
  #include "skArticleOrderDialog.h"
  #include "skTextCell.h"
  #include "pics/ok.xpm"
  #include "pics/nil.xpm"
#endif

#include "skGlobal.h"
#include "skSegorSubSubCategory.h"
#include "skFileIO.h"

#ifdef SEGORKAT_QT

QPixmap *skSegorSubSubCategory::okPixmap = NULL;
QPixmap *skSegorSubSubCategory::nilPixmap = NULL;

#endif

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skSegorSubSubCategory class.
*/
skSegorSubSubCategory::skSegorSubSubCategory() : 
	ID(),
        Description()
{
  
  ;

}

/** Copy constructor for the \a skSegorSubSubCategory class.
* @param elem Category that is copied
*/
skSegorSubSubCategory::skSegorSubSubCategory(const skSegorSubSubCategory &elem) 
{
  ID = elem.ID;
  Description = elem.Description;

  alArticles.clear();
  vector<skSegorArticle>::const_iterator a_it = elem.alArticles.begin();
  for (; a_it != elem.alArticles.end(); ++a_it)
    alArticles.push_back(*a_it);

}

/** Overloaded copy operator ``=''.
* @param elem Category that is copied
*/
skSegorSubSubCategory &skSegorSubSubCategory::operator= (const skSegorSubSubCategory &elem)
{
  if (this == &elem)
    return *this;

  ID = elem.ID;
  Description = elem.Description;

  alArticles.clear();
  vector<skSegorArticle>::const_iterator a_it = elem.alArticles.begin();
  for (; a_it != elem.alArticles.end(); ++a_it)
    alArticles.push_back(*a_it);

  return *this;
}

/** Reads the single entries from the given file stream \a f.
* @param f File stream, the entries are read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorSubSubCategory::read(ifstream &f)
{
  char c;

  alArticles.clear();
  ID = "";
  Description = "";

  // Read ID
  int i;
  for (i = 0; i < 5; ++i)
  {
    f.get(c);
    ID += c;
  }
  // Read description
  bool finished = false;
  for (i = 0; i < 40; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Description += c;
    else
      finished = true;
  }
  simplifyWhiteSpace(Description);

  // Skip additional index
  f.get(c);
  f.get(c);

  return OK;
}


/** Appends the given \a skSegorArticle to the end
* of the list.
* @param article The article to append
*/
void skSegorSubSubCategory::InsertArticle(const skSegorArticle &article)
{
  alArticles.push_back(article);
}

#ifdef SEGORKAT_QT

/** Returns the pixmap for the OK (=ordered) bitmap.
*/
const QPixmap &skSegorSubSubCategory::GetOKPixmap()
{
  if (NULL == okPixmap)
    okPixmap = new QPixmap( okPic );

  return *okPixmap;
}

/** Returns the empty pixmap for articles that are not
* ordered yet.
*/
const QPixmap &skSegorSubSubCategory::GetNilPixmap()
{
  if (NULL == nilPixmap)
    nilPixmap = new QPixmap( nilPic );

  return *nilPixmap;
}

/** Updates the given list view item \a lviItem with
* the article data for ``normal'' display mode.
*/
void skSegorSubSubCategory::UpdateArticleData(Q3ListViewItem *lviItem,
                                              skSegorArticle &article,
                                              int categoryIndex,
                                              int subcategoryIndex,
                                              int subsubcategoryIndex,
                                              int articleIndex)
{
  int vpe;
  int minUnits;
  double minPrice;
  QString statusKey; 
  article.getPriceInfo(&vpe, &minUnits, &minPrice);
  switch (article.Status)
  {
    case 1:  statusKey = " ";
             break;
    case 2:  statusKey = "R";
             break;
    case 3:  statusKey = "B";
             break;
    case 4:  statusKey = "A";
             break;
    default: statusKey = "?";
             break;
  }

  if (article.OrderedUnits > 0)
    lviItem->setPixmap(0, GetOKPixmap());
  else
    lviItem->setPixmap(0, GetNilPixmap());
  lviItem->setText(0, QString(article.Name.c_str()));
  lviItem->setText(1, QString::number(vpe));
  lviItem->setText(2, QString::number(minUnits));
  if (0.0 != minPrice)
  {
    lviItem->setText(3, QString::number(minPrice, 'f', 2));
  }
  else
    lviItem->setText(3, QString(""));

  lviItem->setText(4, statusKey);
  lviItem->setText(5, QString(article.Description.c_str()));
  lviItem->setText(6, QString(article.OtherNames.c_str()));
  lviItem->setText(7, QString::number(categoryIndex));
  lviItem->setText(8, QString::number(subcategoryIndex));
  lviItem->setText(9, QString::number(subsubcategoryIndex));
  lviItem->setText(10, QString::number(articleIndex));
}

/** Updates the given list view item \a lviItem with
* the article data for ``order list'' display mode.
*/
void skSegorSubSubCategory::UpdateOrderData(Q3ListViewItem *lviItem,
                                            skSegorArticle &article,
                                            int categoryIndex,
                                            int subcategoryIndex,
                                            int subsubcategoryIndex,
                                            int articleIndex)
{
  int vpe;
  int minUnits;
  double bruttoPrice;
  double nettoPrice;
  QString statusKey; 
  article.getPriceInfo(&vpe, &minUnits, &bruttoPrice);
  article.getCurrentPrice(&nettoPrice, &bruttoPrice);
  switch (article.Status)
  {
    case 1:  statusKey = " ";
             break;
    case 2:  statusKey = "R";
             break;
    case 3:  statusKey = "B";
             break;
    case 4:  statusKey = "A";
             break;
    default: statusKey = "?";
             break;
  }

  if (article.OrderedUnits > 0)
    lviItem->setPixmap(0, GetOKPixmap());
  else
    lviItem->setPixmap(0, GetNilPixmap());
  lviItem->setText(0, QString(article.Name.c_str()));
  lviItem->setText(1, QString::number(vpe));
  lviItem->setText(2, QString::number(article.OrderedUnits));
  lviItem->setText(3, QString::number(bruttoPrice, 'f', 2));
  lviItem->setText(4, statusKey);
  lviItem->setText(5, QString(article.Description.c_str()));
  lviItem->setText(6, QString(article.OtherNames.c_str()));
  lviItem->setText(7, QString::number(categoryIndex));
  lviItem->setText(8, QString::number(subcategoryIndex));
  lviItem->setText(9, QString::number(subsubcategoryIndex));
  lviItem->setText(10, QString::number(articleIndex));
}

/** Updates the given list view item \a lviItem with
* the article data for ``normal'' display mode.
*/
void skSegorSubSubCategory::UpdateArticleItem(Q3ListViewItem *lviItem,
                                              int categoryIndex,
                                              int subcategoryIndex,
                                              int subsubcategoryIndex,
                                              int articleIndex)
{
  if ((articleIndex < 0) ||
      (((unsigned int) articleIndex) >= alArticles.size()))
    return;

  UpdateArticleData(lviItem, alArticles[articleIndex],
                    categoryIndex, subcategoryIndex,
                    subsubcategoryIndex, articleIndex); 
} 


/** Appends the given \a skSegorArticle to the end
* of the display list in ``normal'' mode.
* @param article The article to append
*/
void skSegorSubSubCategory::AddArticleToListView(Q3ListView *lvParts,
                                                 skSegorArticle &article,
                                                 int categoryIndex,
                                                 int subcategoryIndex,
                                                 int subsubcategoryIndex,
                                                 int articleIndex)
{
  Q3ListViewItem *liNew = new Q3ListViewItem(lvParts);

  UpdateArticleData(liNew, article, categoryIndex, 
                    subcategoryIndex, subsubcategoryIndex, articleIndex);

}


/** Appends the given \a skSegorArticle to the end
* of the display list in ``order list'' mode.
* @param article The article to append
*/
void skSegorSubSubCategory::AddOrderToListView(Q3ListView *lvParts,
                                               skSegorArticle &article,
                                               int categoryIndex,
                                               int subcategoryIndex,
                                               int subsubcategoryIndex,
                                               int articleIndex)
{
  Q3ListViewItem *liNew = new Q3ListViewItem(lvParts);

  UpdateOrderData(liNew, article, categoryIndex, 
                  subcategoryIndex, subsubcategoryIndex, articleIndex);

}

/** Adds all articles of this subsub-category to the
* display list.
*/
void skSegorSubSubCategory::UpdateArticles(Q3ListView *lvParts,
                                           int categoryIndex,
                                           int subcategoryIndex,
                                           int subsubcategoryIndex)
{
  lvParts->clear();

  vector<skSegorArticle>::iterator a_it = alArticles.begin();
  unsigned int aCnt = 0;
  for (; a_it != alArticles.end(); ++a_it, ++aCnt)
    AddArticleToListView(lvParts, *a_it, 
                         categoryIndex, subcategoryIndex, subsubcategoryIndex, aCnt);

}

/** Adds all articles of this subsub-category to the
* display list that match the given search specs.
* @param search The regular expression to search for
* @inName Is ``true'' if the name may match the expr, ``false'' else
* @inDescription Is ``true'' if the description may match the expr, ``false'' else
* @inOtherName Is ``true'' if one of the other article
*              names may match the expr, ``false'' else
* @caseSensitive Is ``true'' if the search is case sensitive, ``false'' else
*/
void skSegorSubSubCategory::SearchArticles(Q3ListView *lvParts,
                                          const QString &search,
                                          bool inName,
                                          bool inDescription,
                                          bool inOtherNames,
                                          bool caseSensitive,
                                          int categoryIndex,
                                          int subcategoryIndex,
                                          int subsubcategoryIndex)
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();
  QString searchFor;
  QString currentString;
  bool found;
  int articleIndex = 0;

  if (false == caseSensitive)
    searchFor = search.lower();
  else
    searchFor = search;

  for (; a_it != alArticles.end(); ++a_it, ++articleIndex)
  {
    // Find search string
    found = false;
    if (true == inName)
    {
      if (false == caseSensitive)
        currentString = QString(a_it->Name.c_str()).lower();
      else
        currentString = QString(a_it->Name.c_str());

      if (currentString.find(QRegExp(searchFor)) >= 0)
        found = true;
    }
    if ((false == found) && (true == inDescription))
    {
      if (false == caseSensitive)
        currentString = QString(a_it->Description.c_str()).lower();
      else
        currentString = QString(a_it->Description.c_str());

      if (currentString.find(QRegExp(searchFor)) >= 0)
        found = true;
    }
    if ((false == found) && (true == inOtherNames))
    {
      if (false == caseSensitive)
        currentString = QString(a_it->OtherNames.c_str()).lower();
      else
        currentString = QString(a_it->OtherNames.c_str());

      if (currentString.find(QRegExp(searchFor)) >= 0)
        found = true;
    }

    if (true == found)
      AddArticleToListView(lvParts, *a_it, categoryIndex, subcategoryIndex,
                           subsubcategoryIndex, articleIndex);
  }

}

/** Opens the ``Article Info'' dialog for the article \a articleIndex.
*/
void skSegorSubSubCategory::ShowArticleInfo(int articleIndex)
{
  if (articleIndex < 0)
    return;
  if ((unsigned int) articleIndex >= alArticles.size())
    return;

  skArticleInfoDialog dlg(alArticles[articleIndex]);

  dlg.exec();
}

/** Opens the ``Order'' dialog for the article \a articleIndex.
*/
void skSegorSubSubCategory::OrderArticle(int articleIndex)
{
  if (articleIndex < 0)
    return;
  if ((unsigned int) articleIndex >= alArticles.size())
    return;

  skArticleOrderDialog dlg(alArticles[articleIndex]);
  if (QDialog::Accepted == dlg.exec())
    alArticles[articleIndex] = dlg.getArticle();
}

/** Adds all ordered articles for this subsub-category to the
* article display list.
*/
void skSegorSubSubCategory::OrderedArticles(Q3ListView *lvParts,
                                            int categoryIndex,
                                            int subcategoryIndex,
                                            int subsubcategoryIndex)
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();
  int articleIndex = 0;

  for (; a_it != alArticles.end(); ++a_it, ++articleIndex)
  {
    // Find ordered article
    if (a_it->OrderedUnits > 0)
      AddOrderToListView(lvParts, *a_it, categoryIndex, subcategoryIndex,
                         subsubcategoryIndex, articleIndex);
  }

}

/** Compiles a RichText/HTML format order list by scanning through
* all articles.
*/
void skSegorSubSubCategory::getHtmlOrderTable(QString &sTable)
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();
  double nettoPrice = 0.0;
  double bruttoPrice = 0.0;

  for (; a_it != alArticles.end(); ++a_it)
  {
    // Find ordered article
    if (a_it->OrderedUnits > 0)
    {
      a_it->getCurrentPrice(&nettoPrice, &bruttoPrice);
      sTable += "<TR><TD ALIGN=RIGHT>" + QString::number(a_it->OrderedUnits);
      sTable += "</TD><TD ALIGN=LEFT>" + QString((a_it->Name).c_str());
      sTable += "</TD><TD ALIGN=RIGHT>" + QString::number(nettoPrice, 'f', 2);
      sTable += "</TD><TD ALIGN=RIGHT>" + QString::number(bruttoPrice, 'f', 2);
      sTable += "</TD></TR>\n";
    }
  }
}
 
/** Compiles a Text format order list by scanning through
* all articles.
*/
void skSegorSubSubCategory::getTextOrderTable(QString &sTable, 
                                              const QString &sepString,
                                              int ColCountWidth, 
                                              int ColArticleWidth, 
                                              int ColNettoWidth,
                                              int ColBruttoWidth)
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();
  double nettoPrice = 0.0;
  double bruttoPrice = 0.0;

  for (; a_it != alArticles.end(); ++a_it)
  {
    // Find ordered article
    if (a_it->OrderedUnits > 0)
    {
      a_it->getCurrentPrice(&nettoPrice, &bruttoPrice);
      addTextCell(sTable, QString::number(a_it->OrderedUnits), ColCountWidth);
      sTable += sepString;
      addTextCell(sTable, (a_it->Name).c_str(), ColArticleWidth, ALIGN_LEFT);
      sTable += sepString;
      addTextCell(sTable, QString::number(nettoPrice, 'f', 2), ColNettoWidth);
      sTable += sepString;
      addTextCell(sTable, QString::number(bruttoPrice, 'f', 2), ColBruttoWidth);
      sTable += "\n";
    }
  }
}


#endif // of: #ifdef SEGORKAT_QT ...

/** Orders the article \a articleIndex by \a orderedUnits
* times, if the latter is zero this means the article
* is removed from the order list. 
* @param articleIndex Index of the article
* @param orderedUnits New number of ordered units
*/
void skSegorSubSubCategory::SetOrderArticle(int articleIndex, int orderedUnits)
{
  if (articleIndex < 0)
    return;
  if ((unsigned int) articleIndex >= alArticles.size())
    return;

  alArticles[articleIndex].OrderedUnits = orderedUnits;

}

/** Removes all articles from the order list.
*/
void skSegorSubSubCategory::DisOrderAll()
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();

  for (; a_it != alArticles.end(); ++a_it)
  {
    // Set ordered units to zero
    a_it->OrderedUnits = 0;
  }
}

/** Updates the price sums and the number of ordered articles.
*/
void skSegorSubSubCategory::UpdateOrderInfo(int *articleCount,
                                            double *nettoSum,
                                            double *bruttoSum)
{
  vector<skSegorArticle>::iterator a_it = alArticles.begin();

  double nettoPrice;
  double bruttoPrice;

  for (; a_it != alArticles.end(); ++a_it)
  {
    // Is this article ordered?
    if (a_it->OrderedUnits > 0)
    {
      // Yes, so add its data to the sums
      *articleCount += 1;
      a_it->getCurrentPrice(&nettoPrice, &bruttoPrice);
      *nettoSum += nettoPrice;
      *bruttoSum += bruttoPrice;
    }
  }
}


